@@ -45,6 +45,7 @@ Handles basic connections to AWS
 
 from __future__ import with_statement
 import base64
+from datetime import datetime
 import errno
 import httplib
 import os
@@ -67,8 +68,10 @@ import boto.handler
 import boto.cacerts
 
 from boto import config, UserAgent
-from boto.exception import AWSConnectionError, BotoClientError
+from boto.exception import AWSConnectionError
+from boto.exception import BotoClientError
 from boto.exception import BotoServerError
+from boto.exception import PleaseRetryException
 from boto.provider import Provider
 from boto.resultset import ResultSet
 
@@ -99,7 +102,7 @@ DEFAULT_CA_CERTS_FILE = os.path.join(os.path.dirname(os.path.abspath(boto.cacert
 class HostConnectionPool(object):
 
     """
-    A pool of connections for one remote (host,is_secure).
+    A pool of connections for one remote (host,port,is_secure).
 
     When connections are added to the pool, they are put into a
     pending queue.  The _mexe method returns connections to the pool
@@ -118,7 +121,7 @@ class HostConnectionPool(object):
 
     Thread Safety:
 
-        This class is used only fram ConnectionPool while it's mutex
+        This class is used only from ConnectionPool while it's mutex
         is held.
     """
 
@@ -143,7 +146,7 @@ class HostConnectionPool(object):
     def get(self):
         """
         Returns the next connection in this pool that is ready to be
-        reused.  Returns None of there aren't any.
+        reused.  Returns None if there aren't any.
         """
         # Discard ready connections that are too old.
         self.clean()
@@ -232,7 +235,7 @@ class ConnectionPool(object):
     STALE_DURATION = 60.0
 
     def __init__(self):
-        # Mapping from (host,is_secure) to HostConnectionPool.
+        # Mapping from (host,port,is_secure) to HostConnectionPool.
         # If a pool becomes empty, it is removed.
         self.host_to_pool = {}
         # The last time the pool was cleaned.
@@ -257,7 +260,7 @@ class ConnectionPool(object):
         """
         return sum(pool.size() for pool in self.host_to_pool.values())
 
-    def get_http_connection(self, host, is_secure):
+    def get_http_connection(self, host, port, is_secure):
         """
         Gets a connection from the pool for the named host.  Returns
         None if there is no connection that can be reused. It's the caller's
@@ -266,18 +269,18 @@ class ConnectionPool(object):
         """
         self.clean()
         with self.mutex:
-            key = (host, is_secure)
+            key = (host, port, is_secure)
             if key not in self.host_to_pool:
                 return None
             return self.host_to_pool[key].get()
 
-    def put_http_connection(self, host, is_secure, conn):
+    def put_http_connection(self, host, port, is_secure, conn):
         """
         Adds a connection to the pool of connections that can be
         reused for the named host.
         """
         with self.mutex:
-            key = (host, is_secure)
+            key = (host, port, is_secure)
             if key not in self.host_to_pool:
                 self.host_to_pool[key] = HostConnectionPool()
             self.host_to_pool[key].put(conn)
@@ -370,7 +373,8 @@ class HTTPRequest(object):
         for key in self.headers:
             val = self.headers[key]
             if isinstance(val, unicode):
-                self.headers[key] = urllib.quote_plus(val.encode('utf-8'))
+                safe = '!"#$%&\'()*+,/:;<=>?@[\\]^`{|}~'
+                self.headers[key] = urllib.quote(val.encode('utf-8'), safe)
 
         connection._auth_handler.add_auth(self, **kwargs)
 
@@ -420,7 +424,7 @@ class AWSAuthConnection(object):
                  https_connection_factory=None, path='/',
                  provider='aws', security_token=None,
                  suppress_consec_slashes=True,
-                 validate_certs=True):
+                 validate_certs=True, profile_name=None):
         """
         :type host: str
         :param host: The host to make the connection to
@@ -431,6 +435,10 @@ class AWSAuthConnection(object):
         :keyword str aws_secret_access_key: Your AWS Secret Access Key
             (provided by Amazon). If none is specified, the value in your
             ``AWS_SECRET_ACCESS_KEY`` environmental variable is used.
+        :keyword str security_token: The security token associated with
+            temporary credentials issued by STS.  Optional unless using
+            temporary credentials.  If none is specified, the environment
+            variable ``AWS_SECURITY_TOKEN`` is used if defined.
 
         :type is_secure: boolean
         :param is_secure: Whether the connection is over SSL
@@ -461,6 +469,10 @@ class AWSAuthConnection(object):
         :type validate_certs: bool
         :param validate_certs: Controls whether SSL certificates
             will be validated or not.  Defaults to True.
+
+        :type profile_name: str
+        :param profile_name: Override usual Credentials section in config
+            file to use a named set of keys instead.
         """
         self.suppress_consec_slashes = suppress_consec_slashes
         self.num_retries = 6
@@ -482,8 +494,16 @@ class AWSAuthConnection(object):
                     "support this feature are not available. Certificate "
                     "validation is only supported when running under Python "
                     "2.6 or later.")
-        self.ca_certificates_file = config.get_value(
+        certs_file = config.get_value(
                 'Boto', 'ca_certificates_file', DEFAULT_CA_CERTS_FILE)
+        if certs_file == 'system':
+            certs_file = None
+        self.ca_certificates_file = certs_file
+        if port:
+            self.port = port
+        else:
+            self.port = PORTS_BY_SECURITY[is_secure]
+
         self.handle_proxy(proxy, proxy_port, proxy_user, proxy_pass)
         # define exceptions from httplib that we want to catch and retry
         self.http_exceptions = (httplib.HTTPException, socket.error,
@@ -511,10 +531,7 @@ class AWSAuthConnection(object):
         if not isinstance(debug, (int, long)):
             debug = 0
         self.debug = config.getint('Boto', 'debug', debug)
-        if port:
-            self.port = port
-        else:
-            self.port = PORTS_BY_SECURITY[is_secure]
+        self.host_header = None
 
         # Timeout used to tell httplib how long to wait for socket timeouts.
         # Default is to leave timeout unchanged, which will in turn result in
@@ -523,9 +540,11 @@ class AWSAuthConnection(object):
         # timeouts will only be applied if Python is 2.6 or greater.
         self.http_connection_kwargs = {}
         if (sys.version_info[0], sys.version_info[1]) >= (2, 6):
-            if config.has_option('Boto', 'http_socket_timeout'):
-                timeout = config.getint('Boto', 'http_socket_timeout')
-                self.http_connection_kwargs['timeout'] = timeout
+            # If timeout isn't defined in boto config file, use 70 second
+            # default as recommended by
+            # http://docs.aws.amazon.com/amazonswf/latest/apireference/API_PollForActivityTask.html
+            self.http_connection_kwargs['timeout'] = config.getint(
+                'Boto', 'http_socket_timeout', 70)
 
         if isinstance(provider, Provider):
             # Allow overriding Provider
@@ -535,19 +554,25 @@ class AWSAuthConnection(object):
             self.provider = Provider(self._provider_type,
                                      aws_access_key_id,
                                      aws_secret_access_key,
-                                     security_token)
+                                     security_token,
+                                     profile_name)
 
-        # allow config file to override default host
+        # Allow config file to override default host, port, and host header.
         if self.provider.host:
             self.host = self.provider.host
+        if self.provider.port:
+            self.port = self.provider.port
+        if self.provider.host_header:
+            self.host_header = self.provider.host_header
 
         self._pool = ConnectionPool()
-        self._connection = (self.server_name(), self.is_secure)
+        self._connection = (self.host, self.port, self.is_secure)
         self._last_rs = None
         self._auth_handler = auth.get_auth_handler(
               host, config, self.provider, self._required_auth_capability())
         if getattr(self, 'AuthServiceName', None) is not None:
             self.auth_service_name = self.AuthServiceName
+        self.request_hook = None
 
     def __repr__(self):
         return '%s:%s' % (self.__class__.__name__, self.host)
@@ -588,13 +613,17 @@ class AWSAuthConnection(object):
     gs_secret_access_key = aws_secret_access_key
     secret_key = aws_secret_access_key
 
+    def profile_name(self):
+        return self.provider.profile_name
+    profile_name = property(profile_name)
+
     def get_path(self, path='/'):
         # The default behavior is to suppress consecutive slashes for reasons
         # discussed at
         # https://groups.google.com/forum/#!topic/boto-dev/-ft0XPUy0y8
         # You can override that behavior with the suppress_consec_slashes param.
         if not self.suppress_consec_slashes:
-            return self.path + re.sub('^/*', "", path)
+            return self.path + re.sub('^(/*)/', "\\1", path)
         pos = path.find('?')
         if pos >= 0:
             params = path[pos:]
@@ -643,7 +672,7 @@ class AWSAuthConnection(object):
         if 'http_proxy' in os.environ and not self.proxy:
             pattern = re.compile(
                 '(?:http://)?' \
-                '(?:(?P<user>\w+):(?P<pass>.*)@)?' \
+                '(?:(?P<user>[\w\-\.]+):(?P<pass>.*)@)?' \
                 '(?P<host>[\w\-\.]+)' \
                 '(?::(?P<port>\d+))?'
             )
@@ -667,60 +696,92 @@ class AWSAuthConnection(object):
             print "http_proxy environment variable does not specify " \
                 "a port, using default"
             self.proxy_port = self.port
-        self.use_proxy = (self.proxy != None)
 
-    def get_http_connection(self, host, is_secure):
-        conn = self._pool.get_http_connection(host, is_secure)
+        self.no_proxy = os.environ.get('no_proxy', '') or os.environ.get('NO_PROXY', '')
+        self.use_proxy = (self.proxy is not None)
+
+    def get_http_connection(self, host, port, is_secure):
+        conn = self._pool.get_http_connection(host, port, is_secure)
         if conn is not None:
             return conn
         else:
-            return self.new_http_connection(host, is_secure)
+            return self.new_http_connection(host, port, is_secure)
 
-    def new_http_connection(self, host, is_secure):
-        if self.use_proxy and not is_secure:
-            host = '%s:%d' % (self.proxy, int(self.proxy_port))
+    def skip_proxy(self, host):
+        if not self.no_proxy:
+            return False
+
+        if self.no_proxy == "*":
+            return True
+
+        hostonly = host
+        hostonly = host.split(':')[0]
+
+        for name in self.no_proxy.split(','):
+            if name and (hostonly.endswith(name) or host.endswith(name)):
+                return True
+
+        return False
+
+    def new_http_connection(self, host, port, is_secure):
         if host is None:
             host = self.server_name()
+
+        # Make sure the host is really just the host, not including
+        # the port number
+        host = host.split(':', 1)[0]
+
+        http_connection_kwargs = self.http_connection_kwargs.copy()
+
+        # Connection factories below expect a port keyword argument
+        http_connection_kwargs['port'] = port
+
+        # Override host with proxy settings if needed
+        if self.use_proxy and not is_secure and \
+                not self.skip_proxy(host):
+            host = self.proxy
+            http_connection_kwargs['port'] = int(self.proxy_port)
+
         if is_secure:
             boto.log.debug(
                     'establishing HTTPS connection: host=%s, kwargs=%s',
-                    host, self.http_connection_kwargs)
-            if self.use_proxy:
+                    host, http_connection_kwargs)
+            if self.use_proxy and not self.skip_proxy(host):
                 connection = self.proxy_ssl(host, is_secure and 443 or 80)
             elif self.https_connection_factory:
                 connection = self.https_connection_factory(host)
             elif self.https_validate_certificates and HAVE_HTTPS_CONNECTION:
                 connection = https_connection.CertValidatingHTTPSConnection(
                         host, ca_certs=self.ca_certificates_file,
-                        **self.http_connection_kwargs)
+                        **http_connection_kwargs)
             else:
                 connection = httplib.HTTPSConnection(host,
-                        **self.http_connection_kwargs)
+                        **http_connection_kwargs)
         else:
             boto.log.debug('establishing HTTP connection: kwargs=%s' %
-                    self.http_connection_kwargs)
+                    http_connection_kwargs)
             if self.https_connection_factory:
                 # even though the factory says https, this is too handy
                 # to not be able to allow overriding for http also.
                 connection = self.https_connection_factory(host,
-                    **self.http_connection_kwargs)
+                    **http_connection_kwargs)
             else:
                 connection = httplib.HTTPConnection(host,
-                    **self.http_connection_kwargs)
+                    **http_connection_kwargs)
         if self.debug > 1:
             connection.set_debuglevel(self.debug)
         # self.connection must be maintained for backwards-compatibility
         # however, it must be dynamically pulled from the connection pool
         # set a private variable which will enable that
         if host.split(':')[0] == self.host and is_secure == self.is_secure:
-            self._connection = (host, is_secure)
+            self._connection = (host, port, is_secure)
         # Set the response class of the http connection to use our custom
         # class.
         connection.response_class = HTTPResponse
         return connection
 
-    def put_http_connection(self, host, is_secure, connection):
-        self._pool.put_http_connection(host, is_secure, connection)
+    def put_http_connection(self, host, port, is_secure, connection):
+        self._pool.put_http_connection(host, port, is_secure, connection)
 
     def proxy_ssl(self, host=None, port=None):
         if host and port:
@@ -730,6 +791,8 @@ class AWSAuthConnection(object):
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         try:
             sock.connect((self.proxy, int(self.proxy_port)))
+            if "timeout" in self.http_connection_kwargs:
+                sock.settimeout(self.http_connection_kwargs["timeout"])
         except:
             raise
         boto.log.debug("Proxy connection: CONNECT %s HTTP/1.0\r\n", host)
@@ -761,9 +824,12 @@ class AWSAuthConnection(object):
         h = httplib.HTTPConnection(host)
 
         if self.https_validate_certificates and HAVE_HTTPS_CONNECTION:
-            boto.log.debug("wrapping ssl socket for proxied connection; "
-                           "CA certificate file=%s",
-                           self.ca_certificates_file)
+            msg = "wrapping ssl socket for proxied connection; "
+            if self.ca_certificates_file:
+                msg += "CA certificate file=%s" %self.ca_certificates_file
+            else:
+                msg += "using system provided SSL certs"
+            boto.log.debug(msg)
             key_file = self.http_connection_kwargs.get('key_file', None)
             cert_file = self.http_connection_kwargs.get('cert_file', None)
             sslSock = ssl.wrap_socket(sock, keyfile=key_file,
@@ -795,6 +861,16 @@ class AWSAuthConnection(object):
         auth = base64.encodestring(self.proxy_user + ':' + self.proxy_pass)
         return {'Proxy-Authorization': 'Basic %s' % auth}
 
+    def set_host_header(self, request):
+        try:
+            request.headers['Host'] = \
+                self._auth_handler.host_header(self.host, request)
+        except AttributeError:
+            request.headers['Host'] = self.host.split(':', 1)[0]
+
+    def set_request_hook(self, hook):
+        self.request_hook = hook
+
     def _mexe(self, request, sender=None, override_num_retries=None,
               retry_handler=None):
         """
@@ -811,6 +887,8 @@ class AWSAuthConnection(object):
         boto.log.debug('Data: %s' % request.body)
         boto.log.debug('Headers: %s' % request.headers)
         boto.log.debug('Host: %s' % request.host)
+        boto.log.debug('Port: %s' % request.port)
+        boto.log.debug('Params: %s' % request.params)
         response = None
         body = None
         e = None
@@ -819,7 +897,8 @@ class AWSAuthConnection(object):
         else:
             num_retries = override_num_retries
         i = 0
-        connection = self.get_http_connection(request.host, self.is_secure)
+        connection = self.get_http_connection(request.host, request.port,
+                                              self.is_secure)
         while i <= num_retries:
             # Use binary exponential backoff to desynchronize client requests.
             next_sleep = random.random() * (2 ** i)
@@ -827,6 +906,14 @@ class AWSAuthConnection(object):
                 # we now re-sign each request before it is retried
                 boto.log.debug('Token: %s' % self.provider.security_token)
                 request.authorize(connection=self)
+                # Only force header for non-s3 connections, because s3 uses
+                # an older signing method + bucket resource URLs that include
+                # the port info. All others should be now be up to date and
+                # not include the port.
+                if 's3' not in self._required_auth_capability():
+                    if not getattr(self, 'anon', False):
+                        self.set_host_header(request)
+                request.start_time = datetime.now()
                 if callable(sender):
                     response = sender(connection, request.method, request.path,
                                       request.body, request.headers)
@@ -849,49 +936,72 @@ class AWSAuthConnection(object):
                             boto.log.debug(msg)
                         time.sleep(next_sleep)
                         continue
-                if response.status == 500 or response.status == 503:
+                if response.status in [500, 502, 503, 504]:
                     msg = 'Received %d response.  ' % response.status
                     msg += 'Retrying in %3.1f seconds' % next_sleep
                     boto.log.debug(msg)
                     body = response.read()
                 elif response.status < 300 or response.status >= 400 or \
                         not location:
-                    self.put_http_connection(request.host, self.is_secure,
-                                             connection)
+                    # don't return connection to the pool if response contains
+                    # Connection:close header, because the connection has been
+                    # closed and default reconnect behavior may do something
+                    # different than new_http_connection. Also, it's probably
+                    # less efficient to try to reuse a closed connection.
+                    conn_header_value = response.getheader('connection')
+                    if conn_header_value == 'close':
+                        connection.close()
+                    else:
+                        self.put_http_connection(request.host, request.port,
+                                                 self.is_secure, connection)
+                    if self.request_hook is not None:
+                        self.request_hook.handle_request_data(request, response)
                     return response
                 else:
                     scheme, request.host, request.path, \
                         params, query, fragment = urlparse.urlparse(location)
                     if query:
                         request.path += '?' + query
+                    # urlparse can return both host and port in netloc, so if
+                    # that's the case we need to split them up properly
+                    if ':' in request.host:
+                        request.host, request.port = request.host.split(':', 1)
                     msg = 'Redirecting: %s' % scheme + '://'
                     msg += request.host + request.path
                     boto.log.debug(msg)
                     connection = self.get_http_connection(request.host,
+                                                          request.port,
                                                           scheme == 'https')
                     response = None
                     continue
+            except PleaseRetryException, e:
+                boto.log.debug('encountered a retry exception: %s' % e)
+                connection = self.new_http_connection(request.host, request.port,
+                                                      self.is_secure)
+                response = e.response
             except self.http_exceptions, e:
                 for unretryable in self.http_unretryable_exceptions:
                     if isinstance(e, unretryable):
                         boto.log.debug(
                             'encountered unretryable %s exception, re-raising' %
                             e.__class__.__name__)
-                        raise e
+                        raise
                 boto.log.debug('encountered %s exception, reconnecting' % \
                                   e.__class__.__name__)
-                connection = self.new_http_connection(request.host,
+                connection = self.new_http_connection(request.host, request.port,
                                                       self.is_secure)
             time.sleep(next_sleep)
             i += 1
         # If we made it here, it's because we have exhausted our retries
         # and stil haven't succeeded.  So, if we have a response object,
         # use it to raise an exception.
-        # Otherwise, raise the exception that must have already h#appened.
+        # Otherwise, raise the exception that must have already happened.
+        if self.request_hook is not None:
+            self.request_hook.handle_request_data(request, response, error=True)
         if response:
             raise BotoServerError(response.status, response.reason, body)
         elif e:
-            raise e
+            raise
         else:
             msg = 'Please report this exception as a Boto Issue!'
             raise BotoClientError(msg)
@@ -901,14 +1011,17 @@ class AWSAuthConnection(object):
         path = self.get_path(path)
         if auth_path is not None:
             auth_path = self.get_path(auth_path)
-        if params == None:
+        if params is None:
             params = {}
         else:
             params = params.copy()
-        if headers == None:
+        if headers is None:
             headers = {}
         else:
             headers = headers.copy()
+        if (self.host_header and
+            not boto.utils.find_matching_headers('host', headers)):
+            headers['host'] = self.host_header
         host = host or self.host
         if self.use_proxy:
             if not auth_path:
@@ -923,13 +1036,14 @@ class AWSAuthConnection(object):
 
     def make_request(self, method, path, headers=None, data='', host=None,
                      auth_path=None, sender=None, override_num_retries=None,
-                     params=None):
+                     params=None, retry_handler=None):
         """Makes a request to the server, with stock multiple-retry logic."""
         if params is None:
             params = {}
         http_request = self.build_base_http_request(method, path, auth_path,
                                                     params, headers, data, host)
-        return self._mexe(http_request, sender, override_num_retries)
+        return self._mexe(http_request, sender, override_num_retries,
+                          retry_handler=retry_handler)
 
     def close(self):
         """(Optional) Close any open HTTP connections.  This is non-destructive,
@@ -948,14 +1062,15 @@ class AWSQueryConnection(AWSAuthConnection):
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None, host=None, debug=0,
                  https_connection_factory=None, path='/', security_token=None,
-                 validate_certs=True):
-        AWSAuthConnection.__init__(self, host, aws_access_key_id,
+                 validate_certs=True, profile_name=None):
+        super(AWSQueryConnection, self).__init__(host, aws_access_key_id,
                                    aws_secret_access_key,
                                    is_secure, port, proxy,
                                    proxy_port, proxy_user, proxy_pass,
                                    debug, https_connection_factory, path,
                                    security_token=security_token,
-                                   validate_certs=validate_certs)
+                                   validate_certs=validate_certs,
+                                   profile_name=profile_name)
 
     def _required_auth_capability(self):
         return []
@@ -966,7 +1081,7 @@ class AWSQueryConnection(AWSAuthConnection):
     def make_request(self, action, params=None, path='/', verb='GET'):
         http_request = self.build_base_http_request(verb, path, None,
                                                     params, {}, '',
-                                                    self.server_name())
+                                                    self.host)
         if action:
             http_request.params['Action'] = action
         if self.APIVersion:
