@@ -28,19 +28,10 @@ from boto.connection import AWSQueryConnection
 from boto.ec2.cloudwatch.metric import Metric
 from boto.ec2.cloudwatch.alarm import MetricAlarm, MetricAlarms, AlarmHistoryItem
 from boto.ec2.cloudwatch.datapoint import Datapoint
-from boto.regioninfo import RegionInfo
+from boto.regioninfo import RegionInfo, get_regions, load_regions
 import boto
 
-RegionData = {
-    'us-east-1': 'monitoring.us-east-1.amazonaws.com',
-    'us-west-1': 'monitoring.us-west-1.amazonaws.com',
-    'us-west-2': 'monitoring.us-west-2.amazonaws.com',
-    'sa-east-1': 'monitoring.sa-east-1.amazonaws.com',
-    'eu-west-1': 'monitoring.eu-west-1.amazonaws.com',
-    'ap-northeast-1': 'monitoring.ap-northeast-1.amazonaws.com',
-    'ap-southeast-1': 'monitoring.ap-southeast-1.amazonaws.com',
-    'ap-southeast-2': 'monitoring.ap-southeast-2.amazonaws.com',
-}
+RegionData = load_regions().get('cloudwatch', {})
 
 
 def regions():
@@ -50,13 +41,7 @@ def regions():
     :rtype: list
     :return: A list of :class:`boto.RegionInfo` instances
     """
-    regions = []
-    for region_name in RegionData:
-        region = RegionInfo(name=region_name,
-                            endpoint=RegionData[region_name],
-                            connection_cls=CloudWatchConnection)
-        regions.append(region)
-    return regions
+    return get_regions('cloudwatch', connection_cls=CloudWatchConnection)
 
 
 def connect_to_region(region_name, **kw_params):
@@ -89,7 +74,7 @@ class CloudWatchConnection(AWSQueryConnection):
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None, debug=0,
                  https_connection_factory=None, region=None, path='/',
-                 security_token=None, validate_certs=True):
+                 security_token=None, validate_certs=True, profile_name=None):
         """
         Init method to create a new connection to EC2 Monitoring Service.
 
@@ -106,17 +91,18 @@ class CloudWatchConnection(AWSQueryConnection):
         if self.region.name == 'eu-west-1':
             validate_certs = False
 
-        AWSQueryConnection.__init__(self, aws_access_key_id,
+        super(CloudWatchConnection, self).__init__(aws_access_key_id,
                                     aws_secret_access_key,
                                     is_secure, port, proxy, proxy_port,
                                     proxy_user, proxy_pass,
                                     self.region.endpoint, debug,
                                     https_connection_factory, path,
                                     security_token,
-                                    validate_certs=validate_certs)
+                                    validate_certs=validate_certs,
+                                    profile_name=profile_name)
 
     def _required_auth_capability(self):
-        return ['ec2']
+        return ['hmac-v4']
 
     def build_dimension_param(self, dimension, params):
         prefix = 'Dimensions.member'
@@ -176,17 +162,17 @@ class CloudWatchConnection(AWSQueryConnection):
                 metric_data['StatisticValues.Minimum'] = s['minimum']
                 metric_data['StatisticValues.SampleCount'] = s['samplecount']
                 metric_data['StatisticValues.Sum'] = s['sum']
-                if value != None:
-                    msg = 'You supplied a value and statistics for a metric.'
-                    msg += 'Posting statistics and not value.'
+                if value is not None:
+                    msg = 'You supplied a value and statistics for a ' + \
+                          'metric.Posting statistics and not value.'
                     boto.log.warn(msg)
-            elif value != None:
+            elif value is not None:
                 metric_data['Value'] = v
             else:
                 raise Exception('Must specify a value or statistics to put.')
 
-            for key, value in metric_data.iteritems():
-                params['MetricData.member.%d.%s' % (index + 1, key)] = value
+            for key, val in metric_data.iteritems():
+                params['MetricData.member.%d.%s' % (index + 1, key)] = val
 
     def get_metric_statistics(self, period, start_time, end_time, metric_name,
                               namespace, statistics, dimensions=None,
@@ -271,9 +257,13 @@ class CloudWatchConnection(AWSQueryConnection):
             pairs that will be used to filter the results.  The key in
             the dictionary is the name of a Dimension.  The value in
             the dictionary is either a scalar value of that Dimension
-            name that you want to filter on, a list of values to
-            filter on or None if you want all metrics with that
-            Dimension name.
+            name that you want to filter on or None if you want all
+            metrics with that Dimension name.  To be included in the
+            result a metric must contain all specified dimensions,
+            although the metric may contain additional dimensions beyond
+            the requested metrics.  The Dimension names, and values must
+            be strings between 1 and 250 characters long. A maximum of
+            10 dimensions are allowed.
 
         :type metric_name: str
         :param metric_name: The name of the Metric to filter against.  If None,
