@@ -28,9 +28,12 @@ import types
 import boto
 import boto.utils
 from boto.ec2.regioninfo import RegionInfo
-from boto.emr.emrobject import JobFlow, RunJobFlowResponse
-from boto.emr.emrobject import AddInstanceGroupsResponse
-from boto.emr.emrobject import ModifyInstanceGroupsResponse
+from boto.emr.emrobject import AddInstanceGroupsResponse, BootstrapActionList, \
+                               Cluster, ClusterSummaryList, HadoopStep, \
+                               InstanceGroupList, InstanceList, JobFlow, \
+                               JobFlowStepList, \
+                               ModifyInstanceGroupsResponse, \
+                               RunJobFlowResponse, StepSummaryList
 from boto.emr.step import JarStep
 from boto.connection import AWSQueryConnection
 from boto.exception import EmrResponseError
@@ -52,22 +55,43 @@ class EmrConnection(AWSQueryConnection):
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None, debug=0,
                  https_connection_factory=None, region=None, path='/',
-                 security_token=None, validate_certs=True):
+                 security_token=None, validate_certs=True, profile_name=None):
         if not region:
             region = RegionInfo(self, self.DefaultRegionName,
                                 self.DefaultRegionEndpoint)
         self.region = region
-        AWSQueryConnection.__init__(self, aws_access_key_id,
+        super(EmrConnection, self).__init__(aws_access_key_id,
                                     aws_secret_access_key,
                                     is_secure, port, proxy, proxy_port,
                                     proxy_user, proxy_pass,
                                     self.region.endpoint, debug,
                                     https_connection_factory, path,
                                     security_token,
-                                    validate_certs=validate_certs)
+                                    validate_certs=validate_certs,
+                                    profile_name=profile_name)
+        # Many of the EMR hostnames are of the form:
+        #     <region>.<service_name>.amazonaws.com
+        # rather than the more common:
+        #     <service_name>.<region>.amazonaws.com
+        # so we need to explicitly set the region_name and service_name
+        # for the SigV4 signing.
+        self.auth_region_name = self.region.name
+        self.auth_service_name = 'elasticmapreduce'
 
     def _required_auth_capability(self):
-        return ['emr']
+        return ['hmac-v4']
+
+    def describe_cluster(self, cluster_id):
+        """
+        Describes an Elastic MapReduce cluster
+
+        :type cluster_id: str
+        :param cluster_id: The cluster id of interest
+        """
+        params = {
+            'ClusterId': cluster_id
+        }
+        return self.get_object('DescribeCluster', params, Cluster)
 
     def describe_jobflow(self, jobflow_id):
         """
@@ -111,6 +135,175 @@ class EmrConnection(AWSQueryConnection):
 
         return self.get_list('DescribeJobFlows', params, [('member', JobFlow)])
 
+    def describe_step(self, cluster_id, step_id):
+        """
+        Describe an Elastic MapReduce step
+
+        :type cluster_id: str
+        :param cluster_id: The cluster id of interest
+        :type step_id: str
+        :param step_id: The step id of interest
+        """
+        params = {
+            'ClusterId': cluster_id,
+            'StepId': step_id
+        }
+
+        return self.get_object('DescribeStep', params, HadoopStep)
+
+    def list_bootstrap_actions(self, cluster_id, marker=None):
+        """
+        Get a list of bootstrap actions for an Elastic MapReduce cluster
+
+        :type cluster_id: str
+        :param cluster_id: The cluster id of interest
+        :type marker: str
+        :param marker: Pagination marker
+        """
+        params = {
+            'ClusterId': cluster_id
+        }
+
+        if marker:
+            params['Marker'] = marker
+
+        return self.get_object('ListBootstrapActions', params, BootstrapActionList)
+
+    def list_clusters(self, created_after=None, created_before=None,
+                      cluster_states=None, marker=None):
+        """
+        List Elastic MapReduce clusters with optional filtering
+
+        :type created_after: datetime
+        :param created_after: Bound on cluster creation time
+        :type created_before: datetime
+        :param created_before: Bound on cluster creation time
+        :type cluster_states: list
+        :param cluster_states: Bound on cluster states
+        :type marker: str
+        :param marker: Pagination marker
+        """
+        params = {}
+        if created_after:
+            params['CreatedAfter'] = created_after.strftime(
+                boto.utils.ISO8601)
+        if created_before:
+            params['CreatedBefore'] = created_before.strftime(
+                boto.utils.ISO8601)
+        if marker:
+            params['Marker'] = marker
+
+        if cluster_states:
+            self.build_list_params(params, cluster_states, 'ClusterStates.member')
+
+        return self.get_object('ListClusters', params, ClusterSummaryList)
+
+    def list_instance_groups(self, cluster_id, marker=None):
+        """
+        List EC2 instance groups in a cluster
+
+        :type cluster_id: str
+        :param cluster_id: The cluster id of interest
+        :type marker: str
+        :param marker: Pagination marker
+        """
+        params = {
+            'ClusterId': cluster_id
+        }
+
+        if marker:
+            params['Marker'] = marker
+
+        return self.get_object('ListInstanceGroups', params, InstanceGroupList)
+
+    def list_instances(self, cluster_id, instance_group_id=None,
+                       instance_group_types=None, marker=None):
+        """
+        List EC2 instances in a cluster
+
+        :type cluster_id: str
+        :param cluster_id: The cluster id of interest
+        :type instance_group_id: str
+        :param instance_group_id: The EC2 instance group id of interest
+        :type instance_group_types: list
+        :param instance_group_types: Filter by EC2 instance group type
+        :type marker: str
+        :param marker: Pagination marker
+        """
+        params = {
+            'ClusterId': cluster_id
+        }
+
+        if instance_group_id:
+            params['InstanceGroupId'] = instance_group_id
+        if marker:
+            params['Marker'] = marker
+
+        if instance_group_types:
+            self.build_list_params(params, instance_group_types,
+                                   'InstanceGroupTypeList.member')
+
+        return self.get_object('ListInstances', params, InstanceList)
+
+    def list_steps(self, cluster_id, step_states=None, marker=None):
+        """
+        List cluster steps
+
+        :type cluster_id: str
+        :param cluster_id: The cluster id of interest
+        :type step_states: list
+        :param step_states: Filter by step states
+        :type marker: str
+        :param marker: Pagination marker
+        """
+        params = {
+            'ClusterId': cluster_id
+        }
+
+        if marker:
+            params['Marker'] = marker
+
+        if step_states:
+            self.build_list_params(params, step_states, 'StepStateList.member')
+
+        return self.get_object('ListSteps', params, StepSummaryList)
+
+    def add_tags(self, resource_id, tags):
+        """
+        Create new metadata tags for the specified resource id.
+
+        :type resource_id: str
+        :param resource_id: The cluster id
+
+        :type tags: dict
+        :param tags: A dictionary containing the name/value pairs.
+                     If you want to create only a tag name, the
+                     value for that tag should be the empty string
+                     (e.g. '') or None.
+        """
+        assert isinstance(resource_id, basestring)
+        params = {
+            'ResourceId': resource_id,
+        }
+        params.update(self._build_tag_list(tags))
+        return self.get_status('AddTags', params, verb='POST')
+
+    def remove_tags(self, resource_id, tags):
+        """
+        Remove metadata tags for the specified resource id.
+
+        :type resource_id: str
+        :param resource_id: The cluster id
+
+        :type tags: list
+        :param tags: A list of tag names to remove.
+        """
+        params = {
+            'ResourceId': resource_id,
+        }
+        params.update(self._build_string_list('TagKeys', tags))
+        return self.get_status('RemoveTags', params, verb='POST')
+
     def terminate_jobflow(self, jobflow_id):
         """
         Terminate an Elastic MapReduce job flow
@@ -150,7 +343,7 @@ class EmrConnection(AWSQueryConnection):
         params.update(self._build_step_list(step_args))
 
         return self.get_object(
-            'AddJobFlowSteps', params, RunJobFlowResponse, verb='POST')
+            'AddJobFlowSteps', params, JobFlowStepList, verb='POST')
 
     def add_instance_groups(self, jobflow_id, instance_groups):
         """
@@ -253,7 +446,7 @@ class EmrConnection(AWSQueryConnection):
 
         :type hadoop_version: str
         :param hadoop_version: Version of Hadoop to use. This no longer
-        defaults to '0.20' and now uses the AMI default.
+            defaults to '0.20' and now uses the AMI default.
 
         :type steps: list(boto.emr.Step)
         :param steps: List of steps to add with the job
@@ -467,6 +660,27 @@ class EmrConnection(AWSQueryConnection):
                 params['Steps.member.%s.%s' % (i+1, key)] = value
         return params
 
+    def _build_string_list(self, field, items):
+        if not isinstance(items, types.ListType):
+            items = [items]
+
+        params = {}
+        for i, item in enumerate(items):
+            params['%s.member.%s' % (field, i + 1)] = item
+        return params
+
+    def _build_tag_list(self, tags):
+        assert isinstance(tags, dict)
+
+        params = {}
+        for i, key_value in enumerate(sorted(tags.iteritems()), start=1):
+            key, value = key_value
+            current_prefix = 'Tags.member.%s' % i
+            params['%s.Key' % current_prefix] = key
+            if value:
+                params['%s.Value' % current_prefix] = value
+        return params
+
     def _build_instance_common_args(self, ec2_keyname, availability_zone,
                                     keep_alive, hadoop_version):
         """
