@@ -18,22 +18,25 @@
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
 # ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
-# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
+# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 #
 
-import xml.sax
-import uuid
+import exception
+import random
 import urllib
+import uuid
+import xml.sax
+
 import boto
 from boto.connection import AWSAuthConnection
 from boto import handler
+import boto.jsonresponse
 from boto.route53.record import ResourceRecordSets
 from boto.route53.zone import Zone
-import boto.jsonresponse
-import exception
+
 
 HZXML = """<?xml version="1.0" encoding="UTF-8"?>
 <CreateHostedZoneRequest xmlns="%(xmlns)s">
@@ -43,7 +46,7 @@ HZXML = """<?xml version="1.0" encoding="UTF-8"?>
     <Comment>%(comment)s</Comment>
   </HostedZoneConfig>
 </CreateHostedZoneRequest>"""
-        
+
 #boto.set_stream_logger('dns')
 
 
@@ -51,21 +54,24 @@ class Route53Connection(AWSAuthConnection):
     DefaultHost = 'route53.amazonaws.com'
     """The default Route53 API endpoint to connect to."""
 
-    Version = '2012-02-29'
+    Version = '2013-04-01'
     """Route53 API version."""
 
-    XMLNameSpace = 'https://route53.amazonaws.com/doc/2012-02-29/'
+    XMLNameSpace = 'https://route53.amazonaws.com/doc/2013-04-01/'
     """XML schema for this Route53 API version."""
 
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None,
                  port=None, proxy=None, proxy_port=None,
                  host=DefaultHost, debug=0, security_token=None,
-                 validate_certs=True):
-        AWSAuthConnection.__init__(self, host,
+                 validate_certs=True, https_connection_factory=None,
+                 profile_name=None):
+        super(Route53Connection, self).__init__(host,
                                    aws_access_key_id, aws_secret_access_key,
                                    True, port, proxy, proxy_port, debug=debug,
                                    security_token=security_token,
-                                   validate_certs=validate_certs)
+                                   validate_certs=validate_certs,
+                                   https_connection_factory=https_connection_factory,
+                                   profile_name=profile_name)
 
     def _required_auth_capability(self):
         return ['route53']
@@ -78,8 +84,9 @@ class Route53Connection(AWSAuthConnection):
                     continue
                 pairs.append(key + '=' + urllib.quote(str(val)))
             path += '?' + '&'.join(pairs)
-        return AWSAuthConnection.make_request(self, action, path,
-                                              headers, data)
+        return super(Route53Connection, self).make_request(action, path,
+                                              headers, data,
+                                              retry_handler=self._retry_handler)
 
     # Hosted Zones
 
@@ -118,7 +125,7 @@ class Route53Connection(AWSAuthConnection):
     def get_hosted_zone(self, hosted_zone_id):
         """
         Get detailed information about a particular Hosted Zone.
-        
+
         :type hosted_zone_id: str
         :param hosted_zone_id: The unique identifier for the Hosted Zone
 
@@ -143,7 +150,7 @@ class Route53Connection(AWSAuthConnection):
 
         :type hosted_zone_name: str
         :param hosted_zone_name: The fully qualified domain name for the Hosted
-        Zone
+            Zone
 
         """
         if hosted_zone_name[-1] != '.':
@@ -158,7 +165,7 @@ class Route53Connection(AWSAuthConnection):
         """
         Create a new Hosted Zone.  Returns a Python data structure with
         information about the newly created Hosted Zone.
-        
+
         :type domain_name: str
         :param domain_name: The name of the domain. This should be a
             fully-specified domain, and should end with a final period
@@ -178,7 +185,7 @@ class Route53Connection(AWSAuthConnection):
             use that.
 
         :type comment: str
-        :param comment: Any comments you want to include about the hosted      
+        :param comment: Any comments you want to include about the hosted
             zone.
 
         """
@@ -204,7 +211,7 @@ class Route53Connection(AWSAuthConnection):
             raise exception.DNSServerError(response.status,
                                            response.reason,
                                            body)
-        
+
     def delete_hosted_zone(self, hosted_zone_id):
         uri = '/%s/hostedzone/%s' % (self.Version, hosted_zone_id)
         response = self.make_request('DELETE', uri)
@@ -219,6 +226,101 @@ class Route53Connection(AWSAuthConnection):
         h.parse(body)
         return e
 
+
+    # Health checks
+
+    POSTHCXMLBody = """<CreateHealthCheckRequest xmlns="%(xmlns)s">
+    <CallerReference>%(caller_ref)s</CallerReference>
+    %(health_check)s
+    </CreateHealthCheckRequest>"""
+
+    def create_health_check(self, health_check, caller_ref=None):
+        """
+        Create a new Health Check
+
+        :type health_check: HealthCheck
+        :param health_check: HealthCheck object
+
+        :type caller_ref: str
+        :param caller_ref: A unique string that identifies the request
+            and that allows failed CreateHealthCheckRequest requests to be retried
+            without the risk of executing the operation twice.  If you don't
+            provide a value for this, boto will generate a Type 4 UUID and
+            use that.
+
+        """
+        if caller_ref is None:
+            caller_ref = str(uuid.uuid4())
+        uri = '/%s/healthcheck' % self.Version
+        params = {'xmlns': self.XMLNameSpace,
+                  'caller_ref': caller_ref,
+                  'health_check': health_check.to_xml()
+                  }
+        xml_body = self.POSTHCXMLBody % params
+        response = self.make_request('POST', uri, {'Content-Type': 'text/xml'}, xml_body)
+        body = response.read()
+        boto.log.debug(body)
+        if response.status == 201:
+            e = boto.jsonresponse.Element()
+            h = boto.jsonresponse.XmlHandler(e, None)
+            h.parse(body)
+            return e
+        else:
+            raise exception.DNSServerError(response.status, response.reason, body)
+
+    def get_list_health_checks(self, maxitems=None, marker=None):
+        """
+        Return a list of health checks
+
+        :type maxitems: int
+        :param maxitems: Maximum number of items to return
+
+        :type marker: str
+        :param marker: marker to get next set of items to list
+
+        """
+
+        params = {}
+        if maxitems is not None:
+            params['maxitems'] = maxitems
+        if marker is not None:
+            params['marker'] = marker
+
+        uri = '/%s/healthcheck' % (self.Version, )
+        response = self.make_request('GET', uri, params=params)
+        body = response.read()
+        boto.log.debug(body)
+        if response.status >= 300:
+            raise exception.DNSServerError(response.status,
+                                           response.reason,
+                                           body)
+        e = boto.jsonresponse.Element(list_marker='HealthChecks', item_marker=('HealthCheck',))
+        h = boto.jsonresponse.XmlHandler(e, None)
+        h.parse(body)
+        return e
+
+    def delete_health_check(self, health_check_id):
+        """
+        Delete a health check
+
+        :type health_check_id: str
+        :param health_check_id: ID of the health check to delete
+
+        """
+        uri = '/%s/healthcheck/%s' % (self.Version, health_check_id)
+        response = self.make_request('DELETE', uri)
+        body = response.read()
+        boto.log.debug(body)
+        if response.status not in (200, 204):
+            raise exception.DNSServerError(response.status,
+                                           response.reason,
+                                           body)
+        e = boto.jsonresponse.Element()
+        h = boto.jsonresponse.XmlHandler(e, None)
+        h.parse(body)
+        return e
+
+
     # Resource Record Sets
 
     def get_all_rrsets(self, hosted_zone_id, type=None,
@@ -226,7 +328,7 @@ class Route53Connection(AWSAuthConnection):
         """
         Retrieve the Resource Record Sets defined for this Hosted Zone.
         Returns the raw XML data returned by the Route53 call.
-        
+
         :type hosted_zone_id: str
         :param hosted_zone_id: The unique identifier for the Hosted Zone
 
@@ -401,3 +503,24 @@ class Route53Connection(AWSAuthConnection):
             if value and not value[-1] == '.':
                 value = "%s." % value
             return value
+
+    def _retry_handler(self, response, i, next_sleep):
+        status = None
+        boto.log.debug("Saw HTTP status: %s" % response.status)
+
+        if response.status == 400:
+            code = response.getheader('Code')
+
+            if code and 'PriorRequestNotComplete' in code:
+                # This is a case where we need to ignore a 400 error, as
+                # Route53 returns this. See
+                # http://docs.aws.amazon.com/Route53/latest/DeveloperGuide/DNSLimitations.html
+                msg = "%s, retry attempt %s" % (
+                    'PriorRequestNotComplete',
+                    i
+                )
+                next_sleep = random.random() * (2 ** i)
+                i += 1
+                status = (msg, i, next_sleep)
+
+        return status
