@@ -27,7 +27,7 @@ import urlparse
 from boto.sqs.message import Message
 
 
-class Queue:
+class Queue(object):
 
     def __init__(self, connection=None, url=None, message_class=Message):
         self.connection = connection
@@ -159,8 +159,8 @@ class Queue:
 
         :type action_name: str or unicode
         :param action_name: The action.  Valid choices are:
-            *|SendMessage|ReceiveMessage|DeleteMessage|
-            ChangeMessageVisibility|GetQueueAttributes
+            SendMessage|ReceiveMessage|DeleteMessage|
+            ChangeMessageVisibility|GetQueueAttributes|*
 
         :rtype: bool
         :return: True if successful, False otherwise.
@@ -182,7 +182,8 @@ class Queue:
         """
         return self.connection.remove_permission(self, label)
 
-    def read(self, visibility_timeout=None, wait_time_seconds=None):
+    def read(self, visibility_timeout=None, wait_time_seconds=None,
+             message_attributes=None):
         """
         Read a single message from the queue.
 
@@ -195,11 +196,17 @@ class Queue:
             If a message is available, the call will return sooner than
             wait_time_seconds.
 
+        :type message_attributes: list
+        :param message_attributes: The name(s) of additional message
+            attributes to return. The default is to return no additional
+            message attributes. Use ``['All']`` or ``['.*']`` to return all.
+
         :rtype: :class:`boto.sqs.message.Message`
         :return: A single message or None if queue is empty
         """
         rs = self.get_messages(1, visibility_timeout,
-                               wait_time_seconds=wait_time_seconds)
+                               wait_time_seconds=wait_time_seconds,
+                               message_attributes=message_attributes)
         if len(rs) == 1:
             return rs[0]
         else:
@@ -216,8 +223,8 @@ class Queue:
         :return: The :class:`boto.sqs.message.Message` object that was written.
         """
         new_msg = self.connection.send_message(self,
-                                               message.get_body_encoded(),
-                                               delay_seconds)
+            message.get_body_encoded(), delay_seconds=delay_seconds,
+            message_attributes=message.message_attributes)
         message.id = new_msg.id
         message.md5 = new_msg.md5
         return message
@@ -231,14 +238,16 @@ class Queue:
             tuple represents a single message to be written
             and consists of and ID (string) that must be unique
             within the list of messages, the message body itself
-            which can be a maximum of 64K in length, and an
+            which can be a maximum of 64K in length, an
             integer which represents the delay time (in seconds)
             for the message (0-900) before the message will
-            be delivered to the queue.
+            be delivered to the queue, and an optional dict of
+            message attributes like those passed to ``send_message``
+            in the connection class.
         """
         return self.connection.send_message_batch(self, messages)
 
-    def new_message(self, body=''):
+    def new_message(self, body='', **kwargs):
         """
         Create new message of appropriate class.
 
@@ -248,13 +257,14 @@ class Queue:
         :rtype: :class:`boto.sqs.message.Message`
         :return: A new Message object
         """
-        m = self.message_class(self, body)
+        m = self.message_class(self, body, **kwargs)
         m.queue = self
         return m
 
     # get a variable number of messages, returns a list of messages
     def get_messages(self, num_messages=1, visibility_timeout=None,
-                     attributes=None, wait_time_seconds=None):
+                     attributes=None, wait_time_seconds=None,
+                     message_attributes=None):
         """
         Get a variable number of messages.
 
@@ -278,13 +288,19 @@ class Queue:
             If a message is available, the call will return sooner than
             wait_time_seconds.
 
+        :type message_attributes: list
+        :param message_attributes: The name(s) of additional message
+            attributes to return. The default is to return no additional
+            message attributes. Use ``['All']`` or ``['.*']`` to return all.
+
         :rtype: list
         :return: A list of :class:`boto.sqs.message.Message` objects.
         """
         return self.connection.receive_message(
             self, number_messages=num_messages,
             visibility_timeout=visibility_timeout, attributes=attributes,
-            wait_time_seconds=wait_time_seconds)
+            wait_time_seconds=wait_time_seconds,
+            message_attributes=message_attributes)
 
     def delete_message(self, message):
         """
