@@ -23,6 +23,7 @@
 import boto
 import os
 import sys
+import textwrap
 from boto.s3.deletemarker import DeleteMarker
 from boto.exception import BotoClientError
 from boto.exception import InvalidUriError
@@ -66,14 +67,11 @@ class StorageUri(object):
 
     def check_response(self, resp, level, uri):
         if resp is None:
-            raise InvalidUriError('Attempt to get %s for "%s" failed.\nThis '
-                                  'can happen if the URI refers to a non-'
-                                  'existent object or if you meant to\noperate '
-                                  'on a directory (e.g., leaving off -R option '
-                                  'on gsutil cp, mv, or ls of a\nbucket). If a '
-                                  'version-ful object was specified, you may '
-                                  'have neglected to\nuse a -v flag.' %
-                                  (level, uri))
+            raise InvalidUriError('\n'.join(textwrap.wrap(
+                'Attempt to get %s for "%s" failed. This can happen if '
+                'the URI refers to a non-existent object or if you meant to '
+                'operate on a directory (e.g., leaving off -R option on gsutil '
+                'cp, mv, or ls of a bucket)' % (level, uri), 80)))
 
     def _check_bucket_uri(self, function_name):
         if issubclass(type(self), BucketStorageUri) and not self.bucket_name:
@@ -103,15 +101,7 @@ class StorageUri(object):
         @return: A connection to storage service provider of the given URI.
         """
         connection_args = dict(self.connection_args or ())
-        # Use OrdinaryCallingFormat instead of boto-default
-        # SubdomainCallingFormat because the latter changes the hostname
-        # that's checked during cert validation for HTTPS connections,
-        # which will fail cert validation (when cert validation is enabled).
-        # Note: the following import can't be moved up to the start of
-        # this file else it causes a config import failure when run from
-        # the resumable upload/download tests.
-        from boto.s3.connection import OrdinaryCallingFormat
-        connection_args['calling_format'] = OrdinaryCallingFormat()
+
         if (hasattr(self, 'suppress_consec_slashes') and
             'suppress_consec_slashes' not in connection_args):
             connection_args['suppress_consec_slashes'] = (
@@ -128,6 +118,23 @@ class StorageUri(object):
                 self.provider_pool[self.scheme] = self.connection
             elif self.scheme == 'gs':
                 from boto.gs.connection import GSConnection
+                # Use OrdinaryCallingFormat instead of boto-default
+                # SubdomainCallingFormat because the latter changes the hostname
+                # that's checked during cert validation for HTTPS connections,
+                # which will fail cert validation (when cert validation is
+                # enabled).
+                #
+                # The same is not true for S3's HTTPS certificates. In fact,
+                # we don't want to do this for S3 because S3 requires the
+                # subdomain to match the location of the bucket. If the proper
+                # subdomain is not used, the server will return a 301 redirect
+                # with no Location header.
+                #
+                # Note: the following import can't be moved up to the
+                # start of this file else it causes a config import failure when
+                # run from the resumable upload/download tests.
+                from boto.s3.connection import OrdinaryCallingFormat
+                connection_args['calling_format'] = OrdinaryCallingFormat()
                 self.connection = GSConnection(access_key_id,
                                                secret_access_key,
                                                **connection_args)
@@ -146,19 +153,6 @@ class StorageUri(object):
                 and ((self.version_id is not None)
                      or (self.generation is not None)))
 
-    def versioned_uri_str(self):
-        """Returns a versionful URI string."""
-        version_desc = ''
-        if not issubclass(type(self), BucketStorageUri):
-          pass
-        elif self.version_id is not None:
-            version_desc += '#' + self.version_id
-        elif self.generation is not None:
-            version_desc += '#' + str(self.generation)
-            if self.meta_generation is not None:
-                version_desc += '.' + str(self.meta_generation)
-        return self.uri + version_desc
-
     def delete_key(self, validate=False, headers=None, version_id=None,
                    mfa_token=None):
         self._check_object_uri('delete_key')
@@ -210,12 +204,20 @@ class StorageUri(object):
 
     def get_contents_to_file(self, fp, headers=None, cb=None, num_cb=10,
                              torrent=False, version_id=None,
-                             res_download_handler=None, response_headers=None):
+                             res_download_handler=None, response_headers=None,
+                             hash_algs=None):
         self._check_object_uri('get_contents_to_file')
         key = self.get_key(None, headers)
         self.check_response(key, 'key', self.uri)
-        key.get_contents_to_file(fp, headers, cb, num_cb, torrent, version_id,
-                                 res_download_handler, response_headers)
+        if hash_algs:
+            key.get_contents_to_file(fp, headers, cb, num_cb, torrent,
+                                     version_id, res_download_handler,
+                                     response_headers,
+                                     hash_algs=hash_algs)
+        else:
+            key.get_contents_to_file(fp, headers, cb, num_cb, torrent,
+                                     version_id, res_download_handler,
+                                     response_headers)
 
     def get_contents_as_string(self, validate=False, headers=None, cb=None,
                                num_cb=10, torrent=False, version_id=None):
@@ -249,8 +251,7 @@ class BucketStorageUri(StorageUri):
 
     def __init__(self, scheme, bucket_name=None, object_name=None,
                  debug=0, connection_args=None, suppress_consec_slashes=True,
-                 version_id=None, generation=None, meta_generation=None,
-                 is_latest=False):
+                 version_id=None, generation=None, is_latest=False):
         """Instantiate a BucketStorageUri from scheme,bucket,object tuple.
 
         @type scheme: string
@@ -258,7 +259,7 @@ class BucketStorageUri(StorageUri):
         @type bucket_name: string
         @param bucket_name: bucket name
         @type object_name: string
-        @param object_name: object name
+        @param object_name: object name, excluding generation/version.
         @type debug: int
         @param debug: debug level to pass in to connection (range 0..2)
         @type connection_args: map
@@ -269,38 +270,66 @@ class BucketStorageUri(StorageUri):
             consecutive slashes will be suppressed in key paths.
         @param version_id: Object version id (S3-specific).
         @param generation: Object generation number (GCS-specific).
-        @param meta_generation: Object meta-generation number (GCS-specific).
         @param is_latest: boolean indicating that a versioned object is the
             current version
 
         After instantiation the components are available in the following
-        fields: uri, scheme, bucket_name, object_name.
+        fields: scheme, bucket_name, object_name, version_id, generation,
+        is_latest, versionless_uri, version_specific_uri, uri.
+        Note: If instantiated without version info, the string representation
+        for a URI stays versionless; similarly, if instantiated with version
+        info, the string representation for a URI stays version-specific. If you
+        call one of the uri.set_contents_from_xyz() methods, a specific object
+        version will be created, and its version-specific URI string can be
+        retrieved from version_specific_uri even if the URI was instantiated
+        without version info.
         """
 
         self.scheme = scheme
         self.bucket_name = bucket_name
         self.object_name = object_name
+        self.debug = debug
         if connection_args:
             self.connection_args = connection_args
         self.suppress_consec_slashes = suppress_consec_slashes
-        if self.bucket_name and self.object_name:
-            self.uri = ('%s://%s/%s' % (self.scheme, self.bucket_name,
-                                        self.object_name))
-        elif self.bucket_name:
-            self.uri = ('%s://%s/' % (self.scheme, self.bucket_name))
-        else:
-            self.uri = ('%s://' % self.scheme)
-        self.debug = debug
-
         self.version_id = version_id
         self.generation = generation and int(generation)
-        self.meta_generation = meta_generation and int(meta_generation)
         self.is_latest = is_latest
+        self.is_version_specific = bool(self.generation) or bool(version_id)
+        self._build_uri_strings()
+
+    def _build_uri_strings(self):
+      if self.bucket_name and self.object_name:
+          self.versionless_uri = '%s://%s/%s' % (self.scheme, self.bucket_name,
+                                                 self.object_name)
+          if self.generation:
+              self.version_specific_uri = '%s#%s' % (self.versionless_uri,
+                                                     self.generation)
+          elif self.version_id:
+              self.version_specific_uri = '%s#%s' % (
+                  self.versionless_uri, self.version_id)
+          if self.is_version_specific:
+              self.uri = self.version_specific_uri
+          else:
+              self.uri = self.versionless_uri
+      elif self.bucket_name:
+          self.uri = ('%s://%s/' % (self.scheme, self.bucket_name))
+      else:
+          self.uri = ('%s://' % self.scheme)
 
     def _update_from_key(self, key):
-      self.version_id = getattr(key, 'version_id', None)
-      self.generation = getattr(key, 'generation', None)
-      self.meta_generation = getattr(key, 'meta_generation', None)
+      self._update_from_values(
+          getattr(key, 'version_id', None),
+          getattr(key, 'generation', None),
+          getattr(key, 'is_latest', None),
+          getattr(key, 'md5', None))
+
+    def _update_from_values(self, version_id, generation, is_latest, md5):
+      self.version_id = version_id
+      self.generation = generation
+      self.is_latest = is_latest
+      self._build_uri_strings()
+      self.md5 = md5
 
     def get_key(self, validate=False, headers=None, version_id=None):
         self._check_object_uri('get_key')
@@ -340,8 +369,9 @@ class BucketStorageUri(StorageUri):
             suppress_consec_slashes=self.suppress_consec_slashes)
 
     def clone_replace_key(self, key):
-        """Instantiate a BucketStorageUri from a Key object while maintaining
-        debug and suppress_consec_slashes values.
+        """Instantiate a BucketStorageUri from the current BucketStorageUri, by
+        replacing the object name with the object name and other metadata found
+        in the given Key object (including generation).
 
         @type key: Key
         @param key: key for the new StorageUri to represent
@@ -349,14 +379,11 @@ class BucketStorageUri(StorageUri):
         self._check_bucket_uri('clone_replace_key')
         version_id = None
         generation = None
-        meta_generation = None
         is_latest = False
         if hasattr(key, 'version_id'):
             version_id = key.version_id
         if hasattr(key, 'generation'):
             generation = key.generation
-        if hasattr(key, 'meta_generation'):
-            meta_generation = key.meta_generation
         if hasattr(key, 'is_latest'):
             is_latest = key.is_latest
 
@@ -368,7 +395,6 @@ class BucketStorageUri(StorageUri):
                 suppress_consec_slashes=self.suppress_consec_slashes,
                 version_id=version_id,
                 generation=generation,
-                meta_generation=meta_generation,
                 is_latest=is_latest)
 
     def get_acl(self, validate=False, headers=None, version_id=None):
@@ -509,7 +535,7 @@ class BucketStorageUri(StorageUri):
 
     def names_bucket(self):
         """Returns True if this URI names a bucket."""
-        return self.names_container()
+        return bool(self.bucket_name) and bool(not self.object_name)
 
     def names_file(self):
         """Returns True if this URI names a file."""
@@ -551,27 +577,48 @@ class BucketStorageUri(StorageUri):
         return provider
 
     def set_acl(self, acl_or_str, key_name='', validate=False, headers=None,
-                version_id=None):
-        """sets or updates a bucket's acl"""
+                version_id=None, if_generation=None, if_metageneration=None):
+        """Sets or updates a bucket's ACL."""
         self._check_bucket_uri('set_acl')
         key_name = key_name or self.object_name or ''
         bucket = self.get_bucket(validate, headers)
         if self.generation:
           bucket.set_acl(
-              acl_or_str, key_name, headers, generation=self.generation)
+              acl_or_str, key_name, headers, generation=self.generation,
+              if_generation=if_generation, if_metageneration=if_metageneration)
         else:
           version_id = version_id or self.version_id
           bucket.set_acl(acl_or_str, key_name, headers, version_id)
 
+    def set_xml_acl(self, xmlstring, key_name='', validate=False, headers=None,
+            version_id=None, if_generation=None, if_metageneration=None):
+        """Sets or updates a bucket's ACL with an XML string."""
+        self._check_bucket_uri('set_xml_acl')
+        key_name = key_name or self.object_name or ''
+        bucket = self.get_bucket(validate, headers)
+        if self.generation:
+          bucket.set_xml_acl(
+              xmlstring, key_name, headers, generation=self.generation,
+              if_generation=if_generation, if_metageneration=if_metageneration)
+        else:
+          version_id = version_id or self.version_id
+          bucket.set_xml_acl(xmlstring, key_name, headers,
+                             version_id=version_id)
+
+    def set_def_xml_acl(self, xmlstring, validate=False, headers=None):
+        """Sets or updates a bucket's default object ACL with an XML string."""
+        self._check_bucket_uri('set_def_xml_acl')
+        self.get_bucket(validate, headers).set_def_xml_acl(xmlstring, headers)
+
     def set_def_acl(self, acl_or_str, validate=False, headers=None,
                     version_id=None):
-        """sets or updates a bucket's default object acl"""
+        """Sets or updates a bucket's default object ACL."""
         self._check_bucket_uri('set_def_acl')
         self.get_bucket(validate, headers).set_def_acl(acl_or_str, headers)
 
     def set_canned_acl(self, acl_str, validate=False, headers=None,
                        version_id=None):
-        """sets or updates a bucket's acl to a predefined (canned) value"""
+        """Sets or updates a bucket's acl to a predefined (canned) value."""
         self._check_object_uri('set_canned_acl')
         self._warn_about_args('set_canned_acl', version_id=version_id)
         key = self.get_key(validate, headers)
@@ -580,8 +627,8 @@ class BucketStorageUri(StorageUri):
 
     def set_def_canned_acl(self, acl_str, validate=False, headers=None,
                            version_id=None):
-        """sets or updates a bucket's default object acl to a predefined
-           (canned) value"""
+        """Sets or updates a bucket's default object acl to a predefined
+           (canned) value."""
         self._check_bucket_uri('set_def_canned_acl ')
         key = self.get_key(validate, headers)
         self.check_response(key, 'key', self.uri)
@@ -622,6 +669,9 @@ class BucketStorageUri(StorageUri):
             result = key.set_contents_from_file(
                 fp, headers, replace, cb, num_cb, policy, md5, size=size,
                 rewind=rewind, res_upload_handler=res_upload_handler)
+            if res_upload_handler:
+                self._update_from_values(None, res_upload_handler.generation,
+                                         None, md5)
         else:
             self._warn_about_args('set_contents_from_file',
                                   res_upload_handler=res_upload_handler)
@@ -645,26 +695,22 @@ class BucketStorageUri(StorageUri):
                  src_version_id=None, storage_class='STANDARD',
                  preserve_acl=False, encrypt_key=False, headers=None,
                  query_args=None, src_generation=None):
+        """Returns newly created key."""
         self._check_object_uri('copy_key')
         dst_bucket = self.get_bucket(validate=False, headers=headers)
         if src_generation:
-          dst_bucket.copy_key(new_key_name=self.object_name,
-                              src_bucket_name=src_bucket_name,
-                              src_key_name=src_key_name, metadata=metadata,
-                              storage_class=storage_class,
-                              preserve_acl=preserve_acl,
-                              encrypt_key=encrypt_key,
-                              headers=headers, query_args=query_args,
-                              src_generation=src_generation)
+            return dst_bucket.copy_key(new_key_name=self.object_name,
+                src_bucket_name=src_bucket_name,
+                src_key_name=src_key_name, metadata=metadata,
+                storage_class=storage_class, preserve_acl=preserve_acl,
+                encrypt_key=encrypt_key, headers=headers, query_args=query_args,
+                src_generation=src_generation)
         else:
-          dst_bucket.copy_key(new_key_name=self.object_name,
-                              src_bucket_name=src_bucket_name,
-                              src_key_name=src_key_name, metadata=metadata,
-                              src_version_id=src_version_id,
-                              storage_class=storage_class,
-                              preserve_acl=preserve_acl,
-                              encrypt_key=encrypt_key,
-                              headers=headers, query_args=query_args)
+            return dst_bucket.copy_key(new_key_name=self.object_name,
+                src_bucket_name=src_bucket_name, src_key_name=src_key_name,
+                metadata=metadata, src_version_id=src_version_id,
+                storage_class=storage_class, preserve_acl=preserve_acl,
+                encrypt_key=encrypt_key, headers=headers, query_args=query_args)
 
     def enable_logging(self, target_bucket, target_prefix=None, validate=False,
                        headers=None, version_id=None):
@@ -677,8 +723,14 @@ class BucketStorageUri(StorageUri):
         bucket = self.get_bucket(validate, headers)
         bucket.disable_logging(headers=headers)
 
+    def get_logging_config(self, validate=False, headers=None, version_id=None):
+        self._check_bucket_uri('get_logging_config')
+        bucket = self.get_bucket(validate, headers)
+        return bucket.get_logging_config(headers=headers)
+
     def set_website_config(self, main_page_suffix=None, error_key=None,
                            validate=False, headers=None):
+        self._check_bucket_uri('set_website_config')
         bucket = self.get_bucket(validate, headers)
         if not (main_page_suffix or error_key):
             bucket.delete_website_configuration(headers)
@@ -686,10 +738,12 @@ class BucketStorageUri(StorageUri):
             bucket.configure_website(main_page_suffix, error_key, headers)
 
     def get_website_config(self, validate=False, headers=None):
+        self._check_bucket_uri('get_website_config')
         bucket = self.get_bucket(validate, headers)
-        return bucket.get_website_configuration_with_xml(headers)
+        return bucket.get_website_configuration(headers)
 
     def get_versioning_config(self, headers=None):
+        self._check_bucket_uri('get_versioning_config')
         bucket = self.get_bucket(False, headers)
         return bucket.get_versioning_status(headers)
 
@@ -705,6 +759,32 @@ class BucketStorageUri(StorageUri):
                                                        preserve_acl,
                                                        headers=headers)
 
+    def compose(self, components, content_type=None, headers=None):
+        self._check_object_uri('compose')
+        component_keys = []
+        for suri in components:
+            component_keys.append(suri.new_key())
+            component_keys[-1].generation = suri.generation
+        self.generation = self.new_key().compose(
+                component_keys, content_type=content_type, headers=headers)
+        self._build_uri_strings()
+        return self
+
+    def get_lifecycle_config(self, validate=False, headers=None):
+        """Returns a bucket's lifecycle configuration."""
+        self._check_bucket_uri('get_lifecycle_config')
+        bucket = self.get_bucket(validate, headers)
+        lifecycle_config = bucket.get_lifecycle_config(headers)
+        self.check_response(lifecycle_config, 'lifecycle', self.uri)
+        return lifecycle_config
+
+    def configure_lifecycle(self, lifecycle_config, validate=False,
+                            headers=None):
+        """Sets or updates a bucket's lifecycle configuration."""
+        self._check_bucket_uri('configure_lifecycle')
+        bucket = self.get_bucket(validate, headers)
+        bucket.configure_lifecycle(lifecycle_config, headers)
+
     def exists(self, headers=None):
       """Returns True if the object exists or False if it doesn't"""
       if not self.object_name:
